package com.nuparu.lumina.entity;

import net.minecraftforge.fml.common.Optional;

import net.minecraft.world.World;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.datafix.DataFixer;
import net.minecraft.util.DamageSource;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.item.ItemStack;
import net.minecraft.entity.projectile.EntityThrowable;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.Entity;

import mod.mcreator.lumina;

import com.nuparu.lumina.utils.ColorRGB;
import com.nuparu.lumina.items.ItemGlowstick;

import com.elytradev.mirage.lighting.Light;
import com.elytradev.mirage.lighting.IEntityLightEventConsumer;
import com.elytradev.mirage.event.GatherLightsEvent;

@Optional.Interface(iface = "com.elytradev.mirage.lighting.ILightEventConsumer", modid = "mirage")
public class EntityGlowstick extends EntityThrowable implements IEntityLightEventConsumer {

	private static final DataParameter<ItemStack> STACK = EntityDataManager.<ItemStack> createKey(EntityGlowstick.class, DataSerializers.ITEM_STACK);

	public EntityGlowstick(World worldIn) {
		super(worldIn);
	}

	public EntityGlowstick(World worldIn, EntityLivingBase throwerIn) {
		super(worldIn, throwerIn);
	}

	public EntityGlowstick(World worldIn, double x, double y, double z) {
		super(worldIn, x, y, z);
	}

	public static void registerFixesSnowball(DataFixer fixer) {
		EntityThrowable.registerFixesThrowable(fixer, "Glowstick");
	}

	@Override
	protected void entityInit() {
		this.dataManager.register(STACK, ItemStack.EMPTY);
	}

	public void setStack(ItemStack stack) {
		this.dataManager.set(STACK, stack);
	}

	public void setStack(NBTTagCompound nbt) {
		this.setStack(new ItemStack(nbt));
	}

	public ItemStack getStack() {
		return this.dataManager.get(STACK);
	}

	@Override
	public void readFromNBT(NBTTagCompound compound) {
		super.readFromNBT(compound);
		if (compound.hasKey("stack")) {
			setStack(compound.getCompoundTag("stack"));
		}
	}

	@Override
	public NBTTagCompound writeToNBT(NBTTagCompound compound) {
		super.writeToNBT(compound);
		compound.setTag("stack", getStack().writeToNBT(new NBTTagCompound()));
		return compound;
	}

	@Override
	protected void onImpact(RayTraceResult result) {
		if (result.entityHit != null) {
			int i = 1;
			result.entityHit.attackEntityFrom(DamageSource.causeThrownDamage(this, this.getThrower()), (float) i);
		}
		if (!this.world.isRemote) {
			this.world.setEntityState(this, (byte) 3);
			this.setDead();
			EntityItem entityItem = new EntityItem(this.world, this.posX, this.posY, this.posZ, getStack());
			this.world.spawnEntity(entityItem);
		}
	}

	@Optional.Method(modid = "mirage")
	@Override
	public void gatherLights(GatherLightsEvent evt, Entity e) {
		ItemStack is = ((EntityGlowstick) e).getStack();
		if (!is.isEmpty() && is.getItem() instanceof ItemGlowstick) {
			ItemGlowstick item = (ItemGlowstick) is.getItem();
			ColorRGB color = item.color;
			evt.add(Light.builder().pos(e).color(color.R, color.G, color.B).radius(5).build());
		}
	}
}
